ham-fisted.profile


current-timesclj

(current-times)

Get the current time map

Get the current time map
raw docstring

reset-times!clj

(reset-times!)

Clear times out of current time map

Clear times out of current time map
raw docstring

time-mscljmacro

(time-ms kw & code)

Time an operation returning the results. Puts time in double milliseconds into the time map.

Time an operation returning the results.  Puts time in double milliseconds
into the time map.
raw docstring

with-timescljmacro

(with-times & code)

Returns {:result :times}. Run a block of code with time map bound to a new map.

Returns {:result :times}. Run a block of code with time map bound to a new map.
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries