(reset-times!)Clear times out of current time map
Clear times out of current time map
(time-ms kw & code)Time an operation returning the results. Puts time in double milliseconds into the time map.
Time an operation returning the results. Puts time in double milliseconds into the time map.
(with-times & code)Returns {:result :times}. Run a block of code with time map bound to a new map.
Returns {:result :times}. Run a block of code with time map bound to a new map.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |