ham-fisted.spliterator

Support for spliterator reduction and parallel reduction.

Support for spliterator reduction and parallel reduction.
raw docstring

->spliteratorclj

(->spliterator ii)

deref-consumercljmacro

(deref-consumer varname accept-code deref-code)

Create a Consumer with support for IDeref

Create a Consumer with support for IDeref
raw docstring

deref-double-consumercljmacro

(deref-double-consumer varname accept-code deref-code)

Create a DoubleConsumer with support for IDeref

Create a DoubleConsumer with support for IDeref
raw docstring

deref-long-consumercljmacro

(deref-long-consumer varname accept-code deref-code)

Create a LongConsumer with support for IDeref

Create a LongConsumer with support for IDeref
raw docstring

split-parallel-reduceclj

(split-parallel-reduce executor-service split ideal-split init-fn rfn merge-fn)

Perform a parallel reduction of a spliterator using the provided ExecutorService

Perform a parallel reduction of a spliterator using the provided ExecutorService
raw docstring

split-reduceclj

(split-reduce rfn split)
(split-reduce rfn acc split)

Reduce over a spliterator. Special support exists for IFn$LLL and IFn$DDD

Reduce over a spliterator.  Special support exists for IFn$LLL and IFn$DDD
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries